@@ -31,7 +31,7 @@ import wx
 
 ##from sysvars import get_color, get_font_string
 
-VERSION = "0.2.5"
+VERSION = "0.2.6"
 
 
 class DefaultConfig(object):
@@ -43,6 +43,10 @@ class DefaultConfig(object):
 
         self.config_version = VERSION
 
+        # Cell calculation timeout in s
+        # -----------------------------
+        self.timeout = repr(10)
+
         # User defined paths
         # ------------------
 
@@ -79,6 +83,7 @@ class DefaultConfig(object):
         self.selection_color = repr(wx.SYS_COLOUR_HIGHLIGHT)
         self.background_color = repr(wx.SYS_COLOUR_WINDOW)
         self.text_color = repr(wx.SYS_COLOUR_WINDOWTEXT)
+        self.freeze_color = repr(wx.SYS_COLOUR_HIGHLIGHT)
 
         # Fonts
 
@@ -143,7 +148,14 @@ class Config(object):
         if key == "version":
             return self.version
 
-        return literal_eval(getattr(self.data, key))
+        try:
+            return literal_eval(getattr(self.data, key))
+
+        except KeyError:
+            # Probably, there is a problem with the config file --> use default
+            setattr(self.data, key, getattr(DefaultConfig(), key))
+
+            return literal_eval(getattr(self.data, key))
 
     def __setitem__(self, key, value):
         """Main config element write access"""
