@@ -47,6 +47,7 @@ import src.lib.xrect as xrect
 from src.model.model import CodeArray
 
 from src.actions._grid_actions import AllGridActions
+from src.gui._grid_cell_editor import GridCellEditor
 
 #use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
@@ -72,13 +73,13 @@ class Grid(wx.grid.Grid, EventMixin):
 
         wx.grid.Grid.__init__(self, main_window, *args, **kwargs)
 
-        self.SetDefaultCellBackgroundColour(wx.Color(255, 255, 255, 255))
+        self.SetDefaultCellBackgroundColour(wx.Colour(255, 255, 255, 255))
 
         # Cursor position on entering selection mode
         self.sel_mode_cursor = None
 
         # Set multi line editor
-        self.SetDefaultEditor(wx.grid.GridCellAutoWrapStringEditor())
+        self.SetDefaultEditor(GridCellEditor(main_window))
 
         # Create new grid
         if S is None:
@@ -124,6 +125,10 @@ class Grid(wx.grid.Grid, EventMixin):
         # The cell that has been selected before the latest selection
         self._last_selected_cell = 0, 0, 0
 
+        # If we are viewing cells based on their frozen status or normally
+        #  (When true, a cross-hatch is displayed for frozen cells)
+        self._view_frozen = False
+
     def _layout(self):
         """Initial layout of grid"""
 
@@ -158,7 +163,6 @@ class Grid(wx.grid.Grid, EventMixin):
         # Grid events
 
         self.GetGridWindow().Bind(wx.EVT_MOTION, handlers.OnMouseMotion)
-        self.Bind(wx.EVT_SCROLLWIN, handlers.OnScroll)
         self.Bind(wx.grid.EVT_GRID_RANGE_SELECT, handlers.OnRangeSelected)
 
         # Context menu
@@ -215,6 +219,7 @@ class Grid(wx.grid.Grid, EventMixin):
 
         # Grid view events
 
+        main_window.Bind(self.EVT_CMD_VIEW_FROZEN, handlers.OnViewFrozen)
         main_window.Bind(self.EVT_CMD_REFRESH_SELECTION,
                          handlers.OnRefreshSelectedCells)
         main_window.Bind(self.EVT_CMD_DISPLAY_GOTO_CELL_DIALOG,
@@ -245,6 +250,7 @@ class Grid(wx.grid.Grid, EventMixin):
 
         main_window.Bind(self.EVT_CMD_SHOW_RESIZE_GRID_DIALOG,
                          handlers.OnResizeGridDialog)
+        main_window.Bind(self.EVT_CMD_QUOTE, handlers.OnQuote)
 
         main_window.Bind(wx.grid.EVT_GRID_ROW_SIZE, handlers.OnRowSize)
         main_window.Bind(wx.grid.EVT_GRID_COL_SIZE, handlers.OnColSize)
@@ -260,100 +266,6 @@ class Grid(wx.grid.Grid, EventMixin):
     # Collison helper functions for grid drawing
     # ------------------------------------------
 
-    def get_visiblecell_slice(self):
-        """Returns a tuple of 3 slices that contanins the visible cells"""
-
-        topleft_x = self.YToRow(self.GetViewStart()[1] * self.ScrollLineX)
-        topleft_y = self.XToCol(self.GetViewStart()[0] * self.ScrollLineY)
-        topleft = (topleft_x, topleft_y)
-        row, col = topleft_x + 1, topleft_y + 1  # Ensures visibility
-
-        while self.IsVisible(row, topleft[1], wholeCellVisible=False):
-            row += 1
-        while self.IsVisible(topleft[0], col, wholeCellVisible=False):
-            col += 1
-        lowerright = (row, col)  # This cell is *not* visible
-        return (slice(topleft[0], lowerright[0]),
-                slice(topleft[1], lowerright[1]),
-                slice(self.current_table, self.current_table + 1))
-
-    def colliding_cells(self, row, col, textbox):
-        """Generates distance, row, col tuples of colliding cells
-
-        Parameters
-        ----------
-        row: Integer
-        \tRow of cell that is tested for collision
-        col: Integer
-        \tColumn of cell that is tested for collision
-        """
-
-        def l1_radius_cells(dist):
-            """Generator of cell index tuples with distance dist to o"""
-
-            if not dist:
-                yield 0, 0
-
-            else:
-                for pos in xrange(-dist, dist + 1):
-                    yield pos, dist
-                    yield pos, -dist
-                    yield dist, pos
-                    yield -dist, pos
-
-        def get_max_visible_distance(row, col):
-            """Returns maximum distance between current and any visible cell"""
-
-            vis_cell_slice = self.get_visiblecell_slice()
-            vis_row_min = vis_cell_slice[0].start
-            vis_row_max = vis_cell_slice[0].stop
-            vis_col_min = vis_cell_slice[1].start
-            vis_col_max = vis_cell_slice[1].stop
-
-            return max(vis_row_max - row, vis_col_max - col,
-                       row - vis_row_min, col - vis_col_min)
-
-        for dist in xrange(get_max_visible_distance(row, col)):
-            all_empty = True
-
-            for radius_cell in l1_radius_cells(dist + 1):
-                __row = radius_cell[0] + row
-                __col = radius_cell[1] + col
-
-                if self.IsVisible(__row, __col, wholeCellVisible=False):
-                    cell_rect = self.CellToRect(__row, __col)
-                    cell_rect = xrect.Rect(cell_rect.x, cell_rect.y,
-                                           cell_rect.width, cell_rect.height)
-                    if textbox.collides_axisaligned_rect(cell_rect):
-                        all_empty = False
-                        yield dist + 1, __row, __col
-
-            # If there are no collisions in a circle, we break
-
-            if all_empty:
-                break
-
-    def get_block_direction(self, rect_row, rect_col, block_row, block_col):
-        """Returns a blocking direction string from UP DOWN RIGHT LEFT"""
-
-        diff_row = rect_row - block_row
-        diff_col = rect_col - block_col
-
-        assert not diff_row == diff_col == 0
-
-        if abs(diff_row) <= abs(diff_col):
-            # Columns are dominant
-            if rect_col < block_col:
-                return "RIGHT"
-            else:
-                return "LEFT"
-        else:
-            # Rows are dominant
-            if rect_row < block_row:
-                return "DOWN"
-            else:
-                return "UP"
-
     def is_merged_cell_drawn(self, key):
         """True if key in merged area shall be drawn
 
@@ -817,21 +729,6 @@ class GridEventHandlers(object):
                 # Ctrl + - pressed
                 post_command_event(self.grid, self.grid.ZoomOutMsg)
 
-            elif keycode == 13:
-            # <Ctrl> + <Enter>
-                grid = self.grid
-                grid.DisableCellEditControl()
-
-                row = self.grid.GetGridCursorRow()
-                col = self.grid.GetGridCursorCol()
-                tab = grid.current_table
-                key = row, col, tab
-
-                val = grid.code_array(key)
-                grid.actions.set_code(key, '"' + val + '"')
-
-                grid.MoveCursorDown(False)
-
         else:
             # No Ctrl pressed
 
@@ -860,11 +757,6 @@ class GridEventHandlers(object):
 
         event.Skip()
 
-    def OnScroll(self, event):
-        """Event handler for grid scroll event"""
-
-        event.Skip()
-
     def OnRangeSelected(self, event):
         """Event handler for grid selection"""
 
@@ -881,6 +773,15 @@ class GridEventHandlers(object):
 
     # Grid view events
 
+    def OnViewFrozen(self, event):
+        """Show cells as frozen status"""
+
+        self.grid._view_frozen = not self.grid._view_frozen
+
+        self.grid.ForceRefresh()
+
+        event.Skip()
+
     def OnDisplayGoToCellDialog(self, event):
         """Shift a given cell into view"""
 
@@ -1228,6 +1129,29 @@ class GridEventHandlers(object):
 
         event.Skip()
 
+    def OnQuote(self, event):
+        """Quotes selection or if none the current cell"""
+
+        grid = self.grid
+        grid.DisableCellEditControl()
+
+        # Is a selection present?
+        if self.grid.IsSelection():
+            # Enclose all selected cells
+            self.grid.actions.quote_selection()
+
+            # Update grid
+            self.grid.ForceRefresh()
+
+        else:
+            row = self.grid.GetGridCursorRow()
+            col = self.grid.GetGridCursorCol()
+            key = row, col, grid.current_table
+
+            self.grid.actions.quote_code(key)
+
+            grid.MoveCursorDown(False)
+
     # Grid attribute events
 
     def OnRowSize(self, event):
@@ -1235,24 +1159,61 @@ class GridEventHandlers(object):
 
         row = event.GetRowOrCol()
         tab = self.grid.current_table
-
         rowsize = self.grid.GetRowSize(row) / self.grid.grid_renderer.zoom
 
-        self.grid.code_array.set_row_height(row, tab, rowsize)
+        # Detect for resizing group of rows
+        rows = self.grid.GetSelectedRows()
+        if len(rows) == 0:
+            rows = [row,]
+
+        # Detect for selection of rows spanning all columns
+        selection = self.grid.selection
+        num_cols = self.grid.code_array.shape[1]-1
+        for box in zip(selection.block_tl, selection.block_br):
+            leftmost_col = box[0][1]
+            rightmost_col = box[1][1]
+            if leftmost_col == 0 and rightmost_col == num_cols:
+                rows += range(box[0][0], box[1][0]+1)
+
+        for row in rows:
+            self.grid.code_array.set_row_height(row, tab, rowsize,
+                                                mark_unredo=False)
+            self.grid.SetRowSize(row, rowsize)
+        self.grid.code_array.unredo.mark()
 
         event.Skip()
+        self.grid.Refresh()
+
 
     def OnColSize(self, event):
         """Column size event handler"""
 
         col = event.GetRowOrCol()
         tab = self.grid.current_table
-
         colsize = self.grid.GetColSize(col) / self.grid.grid_renderer.zoom
 
-        self.grid.code_array.set_col_width(col, tab, colsize)
+        # Detect for resizing group of cols
+        cols = self.grid.GetSelectedCols()
+        if len(cols) == 0:
+            cols = [col,]
+
+        # Detect for selection of rows spanning all columns
+        selection = self.grid.selection
+        num_rows = self.grid.code_array.shape[0]-1
+        for box in zip(selection.block_tl, selection.block_br):
+            top_row = box[0][0]
+            bottom_row = box[1][0]
+            if top_row == 0 and bottom_row == num_rows:
+                cols += range(box[0][1], box[1][1]+1)
+
+        for col in cols:
+            self.grid.code_array.set_col_width(col, tab, colsize,
+                                               mark_unredo=False)
+            self.grid.SetColSize(col, colsize)
+        self.grid.code_array.unredo.mark()
 
         event.Skip()
+        self.grid.Refresh()
 
     # Undo and redo events
 
@@ -1280,4 +1241,4 @@ class GridEventHandlers(object):
         self.grid.update_entry_line()
         self.grid.update_attribute_toolbar()
 
-# End of class GridEventHandlers
+# End of class GridEventHandlers
\ No newline at end of file
