@@ -74,7 +74,7 @@ def get_pen_from_data(pendata):
 
 
 def code2color(color_string):
-    """Returns wx.Colour from 3-tuple of floats in [0.0, 1.0]"""
+    """Returns wx.Colour from a string of a 3-tuple of floats in [0.0, 1.0]"""
 
     color_tuple = ast.literal_eval(color_string)
     color_tuple_int = map(lambda x: int(x * 255.0), color_tuple)
@@ -88,6 +88,18 @@ def color2code(color):
     return unicode(tuple(i / 255.0 for i in color.Get()))
 
 
+def unquote_string(code):
+    """Returns a string from code that contains aa repr of the string"""
+
+    if code[0] in ['"', "'"]:
+        start = 1
+    else:
+        # start may have a Unicode or raw string
+        start = 2
+
+    return code[start:-1]
+
+
 def parse_dict_strings(code):
     """Generator of elements of a dict that is given in the code string
 
@@ -120,4 +132,4 @@ def parse_dict_strings(code):
             yield code[chunk_start: i].strip()
             chunk_start = i + 1
 
-    yield code[chunk_start:i + 1].strip()
\ No newline at end of file
+    yield code[chunk_start:i + 1].strip()
